var __defProp = Object.defineProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};

// src/Adaptor.js
var Adaptor_exports = {};
__export(Adaptor_exports, {
  alterState: () => alterState,
  arrayToString: () => arrayToString,
  combine: () => combine,
  dataPath: () => dataPath,
  dataValue: () => dataValue,
  dateFns: () => dateFns,
  describeTable: () => describeTable,
  each: () => each,
  execute: () => execute,
  field: () => field,
  fields: () => fields,
  findValue: () => findValue,
  fn: () => fn,
  fnIf: () => fnIf,
  group: () => group,
  http: () => http,
  insert: () => insert,
  insertMany: () => insertMany,
  insertTable: () => insertTable,
  lastReferenceValue: () => lastReferenceValue,
  merge: () => merge,
  modifyTable: () => modifyTable,
  sourceValue: () => sourceValue,
  sql: () => sql,
  upsert: () => upsert,
  upsertIf: () => upsertIf,
  upsertMany: () => upsertMany
});
import {
  execute as commonExecute,
  composeNextState
} from "@openfn/language-common";
import { expandReferences } from "@openfn/language-common/util";
import pg from "pg";
import format from "pg-format";
import {
  alterState,
  arrayToString,
  combine,
  dataPath,
  dataValue,
  dateFns,
  each,
  field,
  fields,
  fn,
  fnIf,
  http,
  group,
  lastReferenceValue,
  merge,
  sourceValue
} from "@openfn/language-common";
function execute(...operations) {
  const initialState = {
    references: [],
    data: null,
    queries: []
  };
  return (state) => {
    return commonExecute(
      createClient,
      connect,
      ...operations,
      disconnect,
      cleanupState
    )({ ...initialState, ...state }).catch((e) => {
      console.error(e);
      console.error("Unhandled error in the operations. Exiting process.");
      process.exit(1);
    });
  };
}
function createClient(state) {
  const {
    host,
    port,
    database,
    password,
    user,
    ssl,
    allowSelfSignedCert,
    ca,
    key,
    cert
  } = state.configuration;
  const sslOptions = ssl ? { rejectUnauthorized: !allowSelfSignedCert, cert, ca, key } : false;
  var config = { host, port, database, user, password, ssl: sslOptions };
  var client = new pg.Client(config);
  return { ...state, client };
}
function connect(state) {
  let { client } = state;
  client.connect();
  return { ...state, client };
}
function disconnect(state) {
  let { client } = state;
  client.end();
  return state;
}
function cleanupState(state) {
  delete state.client;
  return state;
}
function escapeRegExp(string) {
  return string.replace(/[.*+?^${}()|[\]\\]/g, "\\$&");
}
function handleValues(sqlString, nullString) {
  let sql2 = sqlString;
  if (nullString == false) {
    return sqlString;
  } else if (Array.isArray(nullString)) {
    nullString.forEach((ns) => {
      const re2 = new RegExp(escapeRegExp(ns), "g");
      sql2 = sql2.replace(re2, "NULL");
    });
    return sql2;
  } else if (typeof nullString === "object") {
    throw "setNull must be a string or an array of strings.";
  }
  const re = new RegExp(escapeRegExp(nullString), "g");
  return sqlString.replace(re, "NULL");
}
function handleOptions(options) {
  if (options && options.setNull === false) {
    return false;
  }
  return options && options.setNull || "'undefined'";
}
function queryHandler(state, query, options, callback) {
  const { client } = state;
  return new Promise((resolve, reject) => {
    if (options) {
      if (options.writeSql) {
        console.log("Adding prepared SQL to state.queries array.");
        state.queries.push(query);
      }
      if (options.execute === false) {
        console.log("Not executing query; options.execute === false");
        resolve("Query not executed.");
        return state;
      }
    }
    client.query(query, (err, result) => {
      if (err) {
        reject(err);
        client.end();
      } else {
        console.log(
          `${result.command} succeeded, rowCount: ${result.rowCount}`
        );
        resolve(result);
      }
    });
  }).then((response) => {
    const nextState = {
      ...composeNextState(state, response.rows),
      response
    };
    if (callback)
      return callback(nextState);
    return nextState;
  });
}
function sql(sqlQuery, options, callback) {
  return (state) => {
    const { client } = state;
    const [resolvedSqlQuery, resolvedOptions] = expandReferences(
      state,
      sqlQuery,
      options
    );
    try {
      console.log("Preparing to execute sql statement");
      return queryHandler(state, resolvedSqlQuery, resolvedOptions, callback);
    } catch (e) {
      client.end();
      throw e;
    }
  };
}
function findValue(filter) {
  return (state) => {
    const { client } = state;
    const [resolvedFilter] = expandReferences(state, filter);
    const {
      uuid,
      relation,
      where: whereData,
      operator: operatorData
    } = resolvedFilter;
    let conditionsArray = [];
    for (let key in whereData)
      conditionsArray.push(
        `${key} ${operatorData ? operatorData[key] : "="} '${whereData[key]}'`
      );
    const condition = conditionsArray.length > 0 ? `where ${conditionsArray.join(" and ")}` : "";
    try {
      const body = `select ${uuid} from ${relation} ${condition}`;
      console.log("Preparing to execute sql statement");
      let returnValue = null;
      return new Promise((resolve, reject) => {
        client.query(body, (err, result) => {
          if (err) {
            console.log(err);
            reject(err);
            client.end();
          } else {
            if (result.rows.length > 0) {
              returnValue = result.rows[0][uuid];
            }
            const nextState = {
              ...composeNextState(state, returnValue),
              result: returnValue
            };
            resolve(nextState);
          }
        });
      });
    } catch (e) {
      throw e;
    }
  };
}
function insert(table, record, options, callback) {
  return (state) => {
    const { client } = state;
    const [resolvedTable, resolvedRecord, resolvedOptions] = expandReferences(
      state,
      table,
      record,
      options
    );
    try {
      const columns = Object.keys(resolvedRecord).sort();
      const columnsList = columns.join(", ");
      const values = columns.map((key) => resolvedRecord[key]);
      const query = handleValues(
        format(
          `INSERT INTO ${resolvedTable} (${columnsList}) VALUES (%L);`,
          values
        ),
        handleOptions(resolvedOptions)
      );
      const safeQuery = `INSERT INTO ${resolvedTable} (${columnsList}) VALUES [--REDACTED--]];`;
      const queryToLog = (resolvedOptions == null ? void 0 : resolvedOptions.logValues) ? query : safeQuery;
      console.log("Preparing to insert via:", queryToLog);
      return queryHandler(state, query, resolvedOptions, callback);
    } catch (e) {
      client.end();
      throw e;
    }
  };
}
function insertMany(table, records, options, callback) {
  return (state) => {
    let { client } = state;
    const [resolvedTable, resolvedRecords, resolvedOptions] = expandReferences(
      state,
      table,
      records,
      options
    );
    try {
      return new Promise((resolve, reject) => {
        if (!resolvedRecords || resolvedRecords.length === 0) {
          console.log("No records provided; skipping insert.");
          resolve(state);
        }
        const columns = Object.keys(resolvedRecords[0]);
        const columnsList = columns.join(", ");
        const valueSets = resolvedRecords.map((x) => Object.values(x));
        const query = handleValues(
          format(
            `INSERT INTO ${resolvedTable} (${columnsList}) VALUES %L;`,
            valueSets
          ),
          handleOptions(resolvedOptions)
        );
        const safeQuery = `INSERT INTO ${resolvedTable} (${columnsList}) VALUES [--REDACTED--]];`;
        const queryToLog = (resolvedOptions == null ? void 0 : resolvedOptions.logValues) ? query : safeQuery;
        console.log("Preparing to insertMany via:", queryToLog);
        resolve(queryHandler(state, query, resolvedOptions, callback));
      });
    } catch (e) {
      client.end();
      throw e;
    }
  };
}
function upsert(table, uuid, record, options, callback) {
  return (state) => {
    const { client } = state;
    const [resolvedTable, resolvedUuid, resolvedRecord, resolvedOptions] = expandReferences(state, table, uuid, record, options);
    try {
      const columns = Object.keys(resolvedRecord).sort();
      const columnsList = columns.join(", ");
      const values = columns.map((key) => resolvedRecord[key]);
      const conflict = resolvedUuid.split(" ").length > 1 ? resolvedUuid : `(${resolvedUuid})`;
      const updateValues = columns.map((key) => {
        return `${key}=excluded.${key}`;
      }).join(", ");
      const insertValues = format(
        `INSERT INTO ${resolvedTable} (${columnsList}) VALUES (%L)`,
        values
      );
      const query = handleValues(
        `${insertValues}
        ON CONFLICT ${conflict}
        DO UPDATE SET ${updateValues};`,
        handleOptions(resolvedOptions)
      );
      const safeQuery = `INSERT INTO ${resolvedTable} (${columnsList}) VALUES [--REDACTED--]
        ON CONFLICT ${conflict}
        DO UPDATE SET ${updateValues};`;
      const queryToLog = (resolvedOptions == null ? void 0 : resolvedOptions.logValues) ? query : safeQuery;
      console.log("Preparing to upsert via:", queryToLog);
      return queryHandler(state, query, resolvedOptions, callback);
    } catch (e) {
      client.end();
      throw e;
    }
  };
}
function upsertIf(logical, table, uuid, record, options, callback) {
  return (state) => {
    const { client } = state;
    const [
      resolvedLogic,
      resolvedTable,
      resolvedUuid,
      resolvedRecord,
      resolvedOptions
    ] = expandReferences(state, logical, table, uuid, record, options);
    try {
      return new Promise((resolve, reject) => {
        if (!resolvedLogic) {
          console.log(`Skipping upsert for ${resolvedUuid}.`);
          resolve(state);
          return state;
        }
        const columns = Object.keys(resolvedRecord).sort();
        const columnsList = columns.join(", ");
        const values = columns.map((key) => resolvedRecord[key]);
        const conflict = resolvedUuid.split(" ").length > 1 ? resolvedUuid : `(${resolvedUuid})`;
        const updateValues = columns.map((key) => {
          return `${key}=excluded.${key}`;
        }).join(", ");
        const insertValues = format(
          `INSERT INTO ${resolvedTable} (${columnsList}) VALUES (%L)`,
          values
        );
        const query = handleValues(
          `${insertValues}
        ON CONFLICT ${conflict}
        DO UPDATE SET ${updateValues};`,
          handleOptions(resolvedOptions)
        );
        const safeQuery = `INSERT INTO ${resolvedTable} (${columnsList}) VALUES [--REDACTED--]
        ON CONFLICT ${conflict}
        DO UPDATE SET ${updateValues};`;
        const queryToLog = (resolvedOptions == null ? void 0 : resolvedOptions.logValues) ? query : safeQuery;
        console.log("Preparing to upsert via:", queryToLog);
        resolve(queryHandler(state, query, resolvedOptions, callback));
      });
    } catch (e) {
      client.end();
      throw e;
    }
  };
}
function upsertMany(table, uuid, data, options, callback) {
  return (state) => {
    const { client } = state;
    const [resolvedTable, resolvedUuid, resolvedData, resolvedOptions] = expandReferences(state, table, uuid, data, options);
    try {
      return new Promise((resolve, reject) => {
        if (!resolvedData || resolvedData.length === 0) {
          console.log("No records provided; skipping upsert.");
          resolve(state);
        }
        const columns = Object.keys(resolvedData[0]);
        const columnsList = columns.join(", ");
        const values = resolvedData.map((x) => Object.values(x));
        const conflict = resolvedUuid.split(" ").length > 1 ? resolvedUuid : `(${resolvedUuid})`;
        const updateValues = columns.map((key) => {
          return `${key}=excluded.${key}`;
        }).join(", ");
        const insertValues = format(
          `INSERT INTO ${resolvedTable} (${columnsList}) VALUES %L`,
          values
        );
        const query = handleValues(
          `${insertValues}
        ON CONFLICT ${conflict}
        DO UPDATE SET ${updateValues};`,
          handleOptions(resolvedOptions)
        );
        const safeQuery = `INSERT INTO ${resolvedTable} (${columnsList}) VALUES [--REDACTED--]
        ON CONFLICT ${conflict}
        DO UPDATE SET ${updateValues};`;
        const queryToLog = (resolvedOptions == null ? void 0 : resolvedOptions.logValues) ? query : safeQuery;
        console.log("Preparing to upsert via:", queryToLog);
        resolve(queryHandler(state, query, resolvedOptions, callback));
      });
    } catch (e) {
      client.end();
      throw e;
    }
  };
}
function describeTable(tableName, options, callback) {
  return (state) => {
    const { client } = state;
    const [resolvedTableName, resolvedOptions] = expandReferences(
      state,
      tableName,
      options
    );
    try {
      const query = `SELECT column_name, udt_name, is_nullable
        FROM information_schema.columns
        WHERE table_name='${resolvedTableName}';`;
      console.log("Preparing to describe table via:", query);
      return queryHandler(state, query, resolvedOptions, callback);
    } catch (e) {
      client.end();
      throw e;
    }
  };
}
function insertTable(tableName, columns, options, callback) {
  return (state) => {
    const { client } = state;
    const [resolvedTableName, resolvedColumns, resolvedOptions] = expandReferences(state, tableName, columns, options);
    try {
      return new Promise((resolve, reject) => {
        if (!resolvedColumns || resolvedColumns.length === 0) {
          console.log("No columns provided; skipping table creation.");
          resolve(state);
        }
        const structureData = resolvedColumns.map(
          (x) => `${x.name} ${x.type} ${x.hasOwnProperty("default") ? x.type.includes("varchar") || x.type.includes("text") || x.type.includes("BIT") ? `DEFAULT '${x.default}'` : `DEFAULT ${x.default}` : ""} ${x.unique ? "UNIQUE" : ""} ${x.identity ? "GENERATED BY DEFAULT AS IDENTITY" : ""} ${x.required ? "NOT NULL" : ""}`
        ).join(", ");
        const query = `CREATE TABLE ${resolvedTableName} (
        ${structureData}
      );`;
        console.log("Preparing to create table via:", query);
        resolve(queryHandler(state, query, resolvedOptions, callback));
      });
    } catch (e) {
      client.end();
      throw e;
    }
  };
}
function modifyTable(tableName, columns, options, callback) {
  return (state) => {
    const { client } = state;
    const [resolvedTableName, resolvedColumns, resolvedOptions] = expandReferences(state, tableName, columns, options);
    try {
      return new Promise((resolve, reject) => {
        if (!resolvedColumns || resolvedColumns.length === 0) {
          console.log("No columns provided; skipping table modification.");
          resolve(state);
        }
        const structureData = resolvedColumns.map(
          (x) => `ADD COLUMN ${x.name} ${x.type} ${x.hasOwnProperty("default") ? x.type.includes("varchar") || x.type.includes("text") || x.type.includes("BIT") ? `DEFAULT '${x.default}'` : `DEFAULT ${x.default}` : ""} ${x.identity ? "GENERATED BY DEFAULT AS IDENTITY" : ""} ${x.required ? "NOT NULL" : ""}`
        ).join(", ");
        const query = `ALTER TABLE ${resolvedTableName} ${structureData};`;
        console.log("Preparing to modify table via:", query);
        resolve(queryHandler(state, query, resolvedOptions, callback));
      });
    } catch (e) {
      client.end();
      throw e;
    }
  };
}

// src/index.js
var src_default = Adaptor_exports;
export {
  alterState,
  arrayToString,
  combine,
  dataPath,
  dataValue,
  dateFns,
  src_default as default,
  describeTable,
  each,
  execute,
  field,
  fields,
  findValue,
  fn,
  fnIf,
  group,
  http,
  insert,
  insertMany,
  insertTable,
  lastReferenceValue,
  merge,
  modifyTable,
  sourceValue,
  sql,
  upsert,
  upsertIf,
  upsertMany
};
