/**
 * Execute a sequence of operations.
 * Wraps `language-common/execute`, and prepends initial state for postgresql.
 * @example
 * execute(
 *   create('foo'),
 *   delete('bar')
 * )(state)
 * @private
 * @param {Operations} operations - Operations to be performed.
 * @returns {Operation}
 */
export function execute(...operations: Operations): Operation;
/**
 * Execute an SQL statement
 * @public
 * @example
 * sql(state => `select(*) from ${state.data.tableName};`, { writeSql: true })
 * @function
 * @param {string} sqlQuery - The SQL query as a string.
 * @param {object} [options] - Optional options argument
 * @param {boolean} [options.writeSql] - A boolean value that specifies whether to log the generated SQL statement. Defaults to false.
 * @param {boolean} [options.execute] - A boolean value that specifies whether to execute the generated SQL statement. Defaults to false.
 * @param {function} callback - (Optional) callback function
 * @returns {Operation}
 */
export function sql(sqlQuery: string, options?: {
    writeSql?: boolean;
    execute?: boolean;
}, callback: Function): Operation;
/**
 * Fetch a uuid key given a condition
 * @public
 * @example
 * findValue({
 *    uuid: 'id',
 *    relation: 'users',
 *    where: { first_name: 'Mamadou' },
 *    operator: { first_name: 'like' }
 *  })
 * @function
 * @param {object} [filter] - A filter object with the lookup table, a uuid and the condition
 * @param {string} [filter.uuid] - The uuid value to search for in the specified relation.
 * @param {string} [filter.relation] - The name of the relation to search for the uuid value.
 * @param {object} [filter.where] - An object that contains key-value pairs to filter the search results.
 * @param {object} [filter.operator] - An object that contains key-value pairs to specify the type of comparison to perform on the where clause.
 * @returns {value}
 */
export function findValue(filter?: {
    uuid?: string;
    relation?: string;
    where?: object;
    operator?: object;
}): value;
/**
 * Insert a record
 * @public
 * @example
 * insert('users', { name: 'Elodie', id: 7 }, { setNull: "'NaN'", logValues: true });
 * @function
 * @param {string} table - The target table
 * @param {object} record - Payload data for the record as a JS object or function
 * @param {object} [options] - Optional options argument
 * @param {string} [options.setNull] - A string value that specifies the behavior for inserting null values.
 * @param {boolean} [options.logValues] - A boolean value that specifies whether to log the inserted values to the console. Defaults to false.
 * @param {boolean} [options.writeSql] - A boolean value that specifies whether to log the generated SQL statement. Defaults to false.
 * @param {boolean} [options.execute] - A boolean value that specifies whether to execute the generated SQL statement. Defaults to false.
 * @param {function} callback - (Optional) callback function
 * @returns {Operation}
 */
export function insert(table: string, record: object, options?: {
    setNull?: string;
    logValues?: boolean;
    writeSql?: boolean;
    execute?: boolean;
}, callback: Function): Operation;
/**
 * Insert many records, using the keys of the first as the column template
 * @public
 * @example
 * insertMany('users', state => state.data.recordArray, { setNull: "'undefined'", logValues: true });
 * @function
 * @param {string} table - The target table
 * @param {array} records - An array or a function that takes state and returns an array
 * @param {object} [options] - Optional options argument
 * @param {string} [options.setNull] - A string value that specifies the behavior for inserting null values.
 * @param {boolean} [options.logValues] - A boolean value that specifies whether to log the inserted values to the console. Defaults to false.
 * @param {boolean} [options.writeSql] - A boolean value that specifies whether to log the generated SQL statement. Defaults to false.
 * @param {boolean} [options.execute] - A boolean value that specifies whether to execute the generated SQL statement. Defaults to false.
 * @param {function} callback - (Optional) callback function
 * @returns {Operation}
 */
export function insertMany(table: string, records: any[], options?: {
    setNull?: string;
    logValues?: boolean;
    writeSql?: boolean;
    execute?: boolean;
}, callback: Function): Operation;
/**
 * Insert or update a record using ON CONFLICT UPDATE
 * @public
 * @example
 * upsert(
 *   'users', // the DB table
 *   'ON CONSTRAINT users_pkey', // a DB column with a unique constraint OR a CONSTRAINT NAME
 *   { name: 'Elodie', id: 7 },
 *   { setNull: ["''", "'undefined'"], writeSql:true, execute: true, logValues: true }
 * )
 * @function
 * @param {string} table - The target table
 * @param {string} uuid - The uuid column to determine a matching/existing record
 * @param {object} record - Payload data for the record as a JS object or function
 * @param {object} [options] - Optional options argument
 * @param {string} [options.setNull] - A string value that specifies the behavior for inserting null values.
 * @param {boolean} [options.writeSql] - A boolean value that specifies whether to log the generated SQL statement. Defaults to false.
 * @param {boolean} [options.execute] - A boolean value that specifies whether to execute the generated SQL statement. Defaults to false.
 * @param {boolean} [options.logValues] - A boolean value that specifies whether to log the inserted values to the console. Defaults to false.
 * @param {function} callback - (Optional) callback function
 * @returns {Operation}
 */
export function upsert(table: string, uuid: string, record: object, options?: {
    setNull?: string;
    writeSql?: boolean;
    execute?: boolean;
    logValues?: boolean;
}, callback: Function): Operation;
/**
 * Insert or update a record based on a logical condition using ON CONFLICT UPDATE
 * @public
 * @example
 * upsertIf(
 *   dataValue('name'),
 *   'users', // the DB table
 *   'ON CONSTRAINT users_pkey', // a DB column with a unique constraint OR a CONSTRAINT NAME
 *   { name: 'Elodie', id: 7 },
 *   { writeSql:true, execute: true }
 * )
 * @function
 * @param {string} logical - a data to check existing value for.
 * @param {string} table - The target table
 * @param {string} uuid - The uuid column to determine a matching/existing record
 * @param {object} record - Payload data for the record as a JS object or function
 * @param {object} [options] - Optional options argument
 * @param {string} [options.setNull] - A string value that specifies the behavior for inserting null values.
 * @param {boolean} [options.writeSql] - A boolean value that specifies whether to log the generated SQL statement. Defaults to false.
 * @param {boolean} [options.execute] - A boolean value that specifies whether to execute the generated SQL statement. Defaults to false.
 * @param {boolean} [options.logValues] - A boolean value that specifies whether to log the inserted values to the console. Defaults to false.
 * @param {function} callback - (Optional) callback function
 * @returns {Operation}
 */
export function upsertIf(logical: string, table: string, uuid: string, record: object, options?: {
    setNull?: string;
    writeSql?: boolean;
    execute?: boolean;
    logValues?: boolean;
}, callback: Function): Operation;
/**
 * Insert or update multiple records using ON CONFLICT UPDATE and excluded
 * @public
 * @example
 * upsertMany(
 *   'users', // the DB table
 *   'email', // a DB column with a unique constraint OR a CONSTRAINT NAME
 *   [
 *     { name: 'one', email: 'one@openfn.org' },
 *     { name: 'two', email: 'two@openfn.org' },
 *   ]
 *  { logValues: true }
 * )
 * @function
 * @param {string} table - The target table
 * @param {string} uuid - The uuid column to determine a matching/existing record
 * @param {array} data - An array of objects or a function that returns an array
 * @param {object} [options] - Optional options argument
 * @param {string} [options.setNull] - A string value that specifies the behavior for inserting null values.
 * @param {boolean} [options.writeSql] - A boolean value that specifies whether to log the generated SQL statement. Defaults to false.
 * @param {boolean} [options.execute] - A boolean value that specifies whether to execute the generated SQL statement. Defaults to false.
 * @param {boolean} [options.logValues] - A boolean value that specifies whether to log the inserted values to the console. Defaults to false.
 * @param {function} callback - (Optional) callback function
 * @returns {Operation}
 */
export function upsertMany(table: string, uuid: string, data: any[], options?: {
    setNull?: string;
    writeSql?: boolean;
    execute?: boolean;
    logValues?: boolean;
}, callback: Function): Operation;
/**
 * List the columns of a table in a database.
 * @public
 * @example
 * describeTable('clinic_visits')
 * @function
 * @param {string} tableName - The name of the table to describe
 * @param {object} [options] - Optional options argument
 * @param {boolean} [options.writeSql] - A boolean value that specifies whether to log the generated SQL statement. Defaults to false.
 * @param {boolean} [options.execute] - A boolean value that specifies whether to execute the generated SQL statement. Defaults to false.
 * @param {function} callback - (Optional) callback function
 * @returns {Operation}
 */
export function describeTable(tableName: string, options?: {
    writeSql?: boolean;
    execute?: boolean;
}, callback: Function): Operation;
/**
 * Create a table in database when given an array of columns and a table_name.
 * @public
 * @example
 * insertTable('table_name', state => state.data.map(
 *   column => ({
 *     name: column.name,
 *     type: column.type,
 *     required: true, // optional
 *     unique: false, // optional - to be set to true for unique constraint
 *   })
 * ));
 * @function
 * @param {string} tableName - The name of the table to create
 * @param {array} columns - An array of form columns
 * @param {object} [options] - Optional options argument
 * @param {boolean} [options.writeSql] - A boolean value that specifies whether to log the generated SQL statement. Defaults to false.
 * @param {boolean} [options.execute] - A boolean value that specifies whether to execute the generated SQL statement. Defaults to false.
 * @param {function} callback - (Optional) callback function
 * @returns {Operation}
 */
export function insertTable(tableName: string, columns: any[], options?: {
    writeSql?: boolean;
    execute?: boolean;
}, callback: Function): Operation;
/**
 * Alter an existing table in the database.
 * @public
 * @example
 * modifyTable('table_name', state => state.data.map(
 *   newColumn => ({
 *     name: newColumn.name,
 *     type: newColumn.type,
 *     required: true, // optional
 *     unique: false, // optional - to be set to true for unique constraint
 *   })
 * ));
 * @function
 * @param {string} tableName - The name of the table to alter
 * @param {array} columns - An array of form columns
 * @param {object} [options] - Optional options argument
 * @param {boolean} [options.writeSql] - A boolean value that specifies whether to log the generated SQL statement. Defaults to false.
 * @param {boolean} [options.execute] - A boolean value that specifies whether to execute the generated SQL statement. Defaults to false.
 * @param {function} callback - (Optional) callback function
 * @returns {Operation}
 */
export function modifyTable(tableName: string, columns: any[], options?: {
    writeSql?: boolean;
    execute?: boolean;
}, callback: Function): Operation;
export { alterState, arrayToString, combine, dataPath, dataValue, dateFns, each, field, fields, fn, fnIf, http, group, lastReferenceValue, merge, sourceValue } from "@openfn/language-common";
